@ECHO OFF
REM playlist converter
REM v0.4 PSR
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION

REM used in temp filename generation
SET "_prefix=playconv"
SET "_suffix=tmp"

REM --------------------------------------------------------------------------------
REM check and process input parameters
REM --------------------------------------------------------------------------------
REM check if any params present, if not show usage message
IF "%~1"=="" (
	CLS
	ECHO=
	ECHO   PLAYLIST CONVERTER SCRIPT
	ECHO  ---------------------------
	ECHO=
	ECHO=
	ECHO  Current Path = %0
	ECHO=
	ECHO=
	ECHO=
	ECHO  Usage:
	ECHO=
	ECHO  "playlist-converter.cmd" direction mode file/directory
	ECHO=
	ECHO  where "direction" is m2p for .m3u to .pla.refs conversion
	ECHO			p2m for .pla.refs to .m3u conversion
	ECHO=
	ECHO  where "mode" is inplace ^| single ^| dir ^| dirrec - see below
	ECHO=
	ECHO  and "file/directory" is the file or directory of files to convert
	ECHO=
	ECHO=
	ECHO=
	ECHO=
	ECHO=
	ECHO=
	GOTO finished
)

REM check direction parameter
IF /I "%~1"=="m2p" GOTO DIRECTION_OK
IF /I "%~1"=="p2m" GOTO DIRECTION_OK
ECHO ERROR: first parameter must specify conversion direction as m2p or p2m
ECHO=
ECHO Type %0 with no parameters to see how to use the command
GOTO unfinished

:DIRECTION_OK
SET "_direction=%~1"

REM check mode
IF "%~2"=="" (
	ECHO ERROR: the second parameter must specify mode of operation.
	ECHO=
	ECHO Type %0 with no parameters to see how to use the command
	GOTO unfinished
)

REM ================================================================================
REM Handle each mode - each subsection below checks the mode parameter, and if
REM it matches does what's necessary for that mode.  Each subsection should end
REM with an goto :finished once finished.  Right at the end is a catch for the mode
REM not matched with any of the handlers below.
REM ================================================================================

REM --------------------------------------------------------------------------------
REM Mode: single and inplace
REM --------------------------------------------------------------------------------
SET "_mode=notset"
IF /I "%~2"=="single" (
	SET "_mode=single"
	SET "_delete=false"
)
IF /I "%~2"=="inplace" (
	SET "_mode=single"
	SET "_delete=true"
)
IF "%_mode%"=="single" (
	REM check file specified
	IF "%~3"=="" (
		ECHO ERROR: the third parameter is missing, you must specify a file to be converted.
		ECHO=
		ECHO Type %0 with no parameters to see how to use the command
		GOTO unfinished
	)
	SET "_file=%~3"

	REM check input file exists
	IF NOT EXIST "!_file!" (
		ECHO=
		ECHO ERROR: The file !_file! does not exist
		ECHO=
		ECHO Type %0 with no parameters to see how to use the command
		GOTO unfinished
	)

	REM check it is not a directory
	REM get file/dir attributes
	SET "_attr=%~a3"
	REM first char is the directory attribute
	SET "_dirattr=!_attr:~0,1!"
	IF "!_dirattr!"=="d" (
		ECHO=
		ECHO ERROR: !_file! is a directory, you must specify a single file instead
		ECHO=
		ECHO Type %0 with no parameters to see how to use the command
		GOTO unfinished
	)

	REM generate output filename
	CALL :GET_NEW_FILENAME "!_file!"

	REM check output file does not exist
	IF EXIST "!_newfile!" (
		ECHO=
		ECHO  ^^!^^! WARNING ^^!^^!
		ECHO  --------------
		ECHO=
		ECHO=
		ECHO  A file already exists with the name:
		ECHO  !_newfile!
		ECHO=
		ECHO  To prevent this file from being overwritten, conversion of the following
		ECHO  playlist has been halted:
		ECHO=
		ECHO  !_file!
		ECHO=
		ECHO=
		GOTO unfinished
	)

	REM check direction and call appropriate converter
	IF /I "%_direction%"=="m2p" CALL :CONVERT_M_TO_P
	IF /I "%_direction%"=="p2m" CALL :CONVERT_P_TO_M
	REM create or delete empty pla file
	IF /I "%_direction%"=="m2p" CALL :CREATE_EMPTY_PLA "!_newfile!"
	IF /I "%_direction%"=="p2m" (
		IF "%_delete%"=="true" CALL :DELETE_EMPTY_PLA "!_file!"
	)

	REM if mode was inplace, delete the input file
	IF "%_delete%"=="true" (
		DEL /A /F "!_file!"
		IF ERRORLEVEL 1 GOTO ERROR_REMOVING_INPUT
	)
	GOTO finished

	:ERROR_REMOVING_INPUT
	ECHO ERROR: could not remove input file !_file! after converting
	ECHO  it to !_newfile!
	GOTO unfinished
)

REM --------------------------------------------------------------------------------
REM Modes: dir and dirrec
REM --------------------------------------------------------------------------------
SET "_mode=notset"
IF "%~2"=="dir" (
	SET "_mode=dir"
	SET "_recurse=no"
)
IF "%~2"=="dirrec" (
	SET "_mode=dir"
	SET "_recurse=yes"
)

IF "%_mode%"=="dir" (
	REM check directory specified
	IF "%~3"=="" (
		ECHO ERROR: third parameter missing, you must specify a directory to convert.
		ECHO=
		ECHO Type %0 with no parameters to see how to use the command
		GOTO unfinished
	)
	SET "_inputdir=%~3"

	REM add trailing backslash if not specified
	SET "_lastchar=!_inputdir:~-1!"
	IF "!_lastchar!" NEQ "\" (
		SET "_inputdir=!_inputdir!\"
	)

	REM check directory exists
	IF NOT EXIST "!_inputdir!" (
		ECHO ERROR: the directory !_inputdir! does not exist
		ECHO.
		ECHO Type %0 with no parameters to see how to use the command
		GOTO unfinished
	)

	REM set input file extension
	IF /I "!_direction!"=="m2p" SET "_ext=.m3u"
	IF /I "!_direction!"=="p2m" SET "_ext=.pla.refs"

	REM recursive version
	IF "%_recurse%"=="yes" (
		REM don't specify path in the file set as this is provided as the /R parameter value
		SET "_fileset=*!_ext!"
		REM /R does not like !var! but is ok with %var%
		REM but can't use %_inputdir% directly or it will not be late-expanded so mods above will not be seen
		SET "_inputdir2=!_inputdir!"
		FOR /R "%_inputdir2%" %%F IN ("!_fileset!") DO (
			SET "_file=%%F"
			ECHO Converting: !_file!

			REM generate output filename
			CALL :GET_NEW_FILENAME "!_file!"

			ECHO To: !_newfile!

			REM check destination file does not exist - if it does, warn and skip
			IF EXIST "!_newfile!" (
				ECHO=
				ECHO  ^^!^^! WARNING ^^!^^!
				ECHO  --------------
				ECHO=
				ECHO=
				ECHO  A file already exists with the name:
				ECHO  !_newfile!
				ECHO=
				ECHO  To prevent this file from being overwritten, conversion of the following
				ECHO  playlist was skipped:
				ECHO=
				ECHO  !_file!
				ECHO=
				ECHO=
				GOTO unfinished
			) ELSE (
				REM check direction and call appropriate converter
				IF /I "!_direction!"=="m2p" (
					CALL :CONVERT_M_TO_P
					CALL :CREATE_EMPTY_PLA "!_newfile!"
				)
				IF /I "!_direction!"=="p2m" CALL :CONVERT_P_TO_M
			)
		)
	)
	REM non recursive
	IF "!_recurse!"=="no" (
		REM include path in the file set
		SET "_fileset=!_inputdir!*!_ext!"
		FOR %%F IN ("!_fileset!") DO (
			SET "_file=%%F"
			ECHO Converting: !_file!

			REM generate output filename
			CALL :GET_NEW_FILENAME "!_file!"

			ECHO To: !_newfile!
			REM check destination file does not exist - if it does, warn and skip
			IF EXIST "!_newfile!" (
				ECHO=
				ECHO  ^^!^^! WARNING ^^!^^!
				ECHO  --------------
				ECHO=
				ECHO=
				ECHO  A file already exists with the name:
				ECHO  !_newfile!
				ECHO=
				ECHO  To prevent this file from being overwritten, conversion of the following
				ECHO  playlist was skipped:
				ECHO=
				ECHO  !_file!
				ECHO=
				ECHO=
				GOTO unfinished
			) ELSE (
				REM check direction and call appropriate converter
				IF /I "!_direction!"=="m2p" CALL :CONVERT_M_TO_P
				IF /I "!_direction!"=="m2p" CALL :CREATE_EMPTY_PLA "!_newfile!"
				IF /I "!_direction!"=="p2m" CALl :CONVERT_P_TO_M
			)
		)
	)

	GOTO :finished
)

REM --------------------------------------------------------------------------------
REM unknown mode - did not match one of the above
ECHO ERROR: second parameter %2 does not specify a valid mode of operation.
ECHO=
ECHO Type %0 with no parameters to see how to use the command
GOTO finished
REM --------------------------------------------------------------------------------

REM ================================================================================
REM finished
REM ================================================================================
REM all successfully completed commands should goto :finished
REM rather than exit or exit /b
REM so we can insert debugging code here (eg pause so we can see messages)
:finished
REM pause finished
EXIT /B

REM ================================================================================
REM unfinished
REM ================================================================================
REM all code that generates an error and fails to complete should goto :unfinished
REM rather than :finished
REM to automatically display console so that the error message can be seen.
:unfinished
PAUSE unfinished
EXIT /B

REM ================================================================================
REM misc utility subroutines
REM ================================================================================

REM -----------------------------------
:GET_NEW_FILENAME
REM -----------------------------------
REM based on _direction
REM generate name of new file from old

IF /I "!_direction!"=="m2p" (
	REM from .m3u to .pla.refs
	REM if original filename has .m3u extension, strip it,
	REM and append .pla.refs; if not just append .pla.refs

	REM input, removing any quotes
	SET "_inp=%~1"
	REM get last 4 chars
	SET "_last=!_inp:~-4!"
	IF "!_last!"==".m3u" (
		SET "_newfile=%~dpn1.pla.refs"
	) ELSE SET "_newfile=!_inp!.pla.refs"
)
IF /I "!_direction!"=="p2m" (
	REM from .pla.refs to .m3u
	REM if original filename has .pla.refs extension, strip it,
	REM and append .m3u; if not just append .m3u

	REM input, without any quotes
	SET "_inp=%~1"
	REM get last 9 chars
	SET "_last=!_inp:~-9!"
	IF "!_last!"==".pla.refs" (
		REM can't use set here, as filename/extension expansions only work in paramater variables
		CALL :STRIP_EXTENSION "%~1"
		CALL :STRIP_EXTENSION "!_stripped!"
		SET "_newfile=!_stripped!.m3u"
	) ELSE SET "_newfile=!_inp!.m3u"
)
GOTO :EOF

REM -----------------------------------
:STRIP_EXTENSION
REM -----------------------------------
REM usage
REM CALL :STRIP_EXTENSION "some file.extension"
REM returns value in _stripped
REM strip extension from a filename
REM so file.one.ext becomes file.one
REM note file becomes file
REM
REM this was written as the %~dpn1% syntax in set
REM only works for parameter-type variables, ie %1, and
REM not local variables eg !_var! or %_var!
SET "_stripped=%~dpn1"
GOTO :EOF

REM -----------------------------------
:GET_TEMP_FILENAME
REM -----------------------------------
REM get name of file to use as a temporary file
REM (does not create the file)
REM generated using %RANDOM%, but if file exists already then tries again
REM with new random numbers
REM files prefixed by _prefix and extension is _suffix
REM eg _prefix=test _suffix=tmp could generate a file named test1234512345.tmp
REM files generated in directory _tempdir  (nb, include trailing \)
REM returns result as _tempfile, note this does *not* include the directory

REM prefix and suffix default to tmp
IF "%_suffix%"=="" SET "_suffix=tmp"
IF "%_prefix%"=="" SET "_prefix=tmp"
REM temp dir defaults to system one
IF "%_tempdir%"=="" SET "_tempdir=%TEMP%\"

:get_temp_filename_again
SET "_tempfile=%_prefix%%RANDOM%%RANDOM%.%_suffix%"
IF EXIST "%_tempdir%%_tempfile%" GOTO get_temp_filename_again
GOTO :EOF

REM -----------------------------------
:CREATE_EMPTY_PLA
REM -----------------------------------
REM parameter - name of .pla.refs file
REM NB no check is made on this filename!
REM this only works correctly if it is a .pla.refs file
REM strip extension - get drive, path, name
REM file.pla.refs -> file.pla
SET "_emptypla=%~dpn1"
TYPE NUL>"%_emptypla%"
GOTO :EOF

REM -----------------------------------
:DELETE_EMPTY_PLA
REM -----------------------------------
REM parameter - name of .pla.refs file
REM NB no check is made on this filename!
REM this only works correctly if it is a .pla.refs file
REM strip extension - get drive, path, name
REM file.pla.refs -> file.pla
SET "_emptypla=%~dpn1"
IF EXIST "%_emptypla%" (
	DEL /A /F "%_emptypla%"
	IF ERRORLEVEL 1 ECHO WARNING: could not delete file %_emptypla%
)
GOTO :EOF

REM ================================================================================
REM subroutines for single file conversions
REM ================================================================================

REM -----------------------------------
:CONVERT_M_TO_P
REM -----------------------------------
REM m3u to pla.refs
REM do each line in the file
REM note - using "TYPE ...|FIND" otherwise blank lines are skipped by "FOR", so can't output blank lines for these
FOR /F "TOKENS=1,* DELIMS=]" %%A IN ('"TYPE "!_file!"|FIND /N /V """') DO (
	REM B not A, A is the line number
	SET "_line=%%B"
	REM skip blank lines
	IF DEFINED _line (
		REM skip #EXT  (nb 4 is length of #EXT)
		SET "_start=!_line:~0,4!"
		IF /I "!_start!" NEQ "#EXT" (
			REM drive test - get 2nd two chars (from position 1, zero-indexed)
			SET "_drivetest=!_line:~1,2!"
			IF "!_drivetest!"==":\" (
				REM /mmc:1:/ followed by all text from position 3 (zero-indexed)
				SET "_line=/mmc:1:/!_line:~3!"
			)
			REM replace backslash with forward
			SET "_line=!_line:\=/!"
			REM output
			>>"!_newfile!" ECHO !_line!
		) ELSE (
			REM lines starting with #EXT - do nothing, could instead output a blank line by un-rem-ing next line
			REM >>"!_newfile!" ECHO=
		)
	) ELSE (
		REM blank lines in original are replicated
		>>"!_newfile!" ECHO=
	)
)
REM return
GOTO :EOF
REM -----------------------------------

REM -----------------------------------
:CONVERT_P_TO_M
REM -----------------------------------
REM pla.refs to m3u
REM -----------------------------------

REM do each line in the file
REM note - using "TYPE ...|FIND" otherwise blank lines are skipped by "FOR", so can't output blank lines for these
FOR /F "TOKENS=1,* DELIMS=]" %%A IN ('"TYPE "!_file!"|FIND /N /V """') DO (
	REM B not A, A is the line number
	SET "_line=%%B"
	REM skip blank lines
	IF DEFINED _line (
		REM mmc test - get first 8 chars (length of /mmc:1:/)
		SET "_mmctest=!_line:~0,8!"
		IF "!_mmctest!"=="/mmc:1:/" (
			REM drive plus all from char 8 onwards (zero indexed)
			SET "_line=F:\!_line:~8!"
		)
	REM replace forward with back slashes
	SET "_line=!_line:/=\!"
	REM output
	>>"!_newfile!" ECHO !_line!
	) ELSE (
		REM blank lines in original are replicated
		>>"!_newfile!" ECHO=
	)
)
GOTO :EOF
